/*
 *   C3Mapper - create minimaps from Caesar 3 scenarios and saved games
 *   Copyright (C) 2007  Bianca van Schaik
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
#include "pngimage.h"
#include "wx/image.h"
#include "wx/palette.h"
#include <iostream>

PNGImage::PNGImage(int width, int height) {
	if (height <= 0 || width <= 0) {
		throw "Invalid size";
	}
	this->width = width;
	this->height = height;
	image = new wxImage(width, height);
}

PNGImage::~PNGImage() {
	delete image;
	colours.clear();
}

void PNGImage::setRGB(int x, int y, int color) {
	setRGB(x, y, (color >> 16) & 0xff, (color >> 8) & 0xff, color & 0xff);
}

void PNGImage::setRGB(int x, int y, int r, int g, int b) {
	// Store which colour is used
	colours.insert((r << 16) | (g << 8) | b);
	image->SetRGB(x, y, r, g, b);
}

bool PNGImage::write(wxString filename) {

#if wxUSE_PALETTE
	int num_colours = colours.size();
	if (num_colours <= 255) {
		wxPalette palette;
		unsigned char *red   = new unsigned char[num_colours+1];
		unsigned char *green = new unsigned char[num_colours+1];
		unsigned char *blue  = new unsigned char[num_colours+1];
		// Fill in bg colour
		red[0] = green[0] = blue[0] = 0;
		
		// Add palette colours to char arrays
		ColourSet::iterator si;
		int i = 1;
		for (si = colours.begin(); si != colours.end(); si++, i++) {
			red[i] = (*si) >> 16;
			green[i] = (*si & 0xff00) >> 8;
			blue[i] = (*si) & 0xff;
		}
		// Create palette & assign it to the image
		palette.Create(num_colours+1, red, green, blue);
		image->SetPalette(palette);
		// Set image format to palette-based
		image->SetOption(wxIMAGE_OPTION_PNG_FORMAT, wxPNG_TYPE_PALETTE);
		delete red;
		delete green;
		delete blue;
	}
#endif
	return image->SaveFile(filename, wxBITMAP_TYPE_PNG);
}

