/*
 *   PharaohMapper - create minimaps from Pharaoh scenarios and saved games
 *   Copyright (C) 2007  Bianca van Schaik
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
#ifndef pngimage_h
#define pngimage_h

#include "wx/image.h"
#include <string>
#include "wx/hashset.h"

WX_DECLARE_HASH_SET( int, wxIntegerHash, wxIntegerEqual, ColourSet );

/**
* Toolkit-independent implementation for writing a PNG image
*/
class PNGImage {
	public:
		/**
		* Create a new image of size width x height
		*/
		PNGImage(int width, int height);
		~PNGImage();
		
		/**
		* Sets pixel (x, y) to color `color'.
		* Color is composed like this: 0xrrggbb
		*/
		void setRGB(int x, int y, int color);
		
		/**
		* Sets pixel (x, y) to the specified colour
		* Only the lower 8 bits of r, g, and b is used
		*/
		void setRGB(int x, int y, int r, int g, int b);
		
		/**
		* Writes the PNG image to `filename' as 8-bit indexed PNG.
		* Returns FALSE if anything fails, TRUE if the file was
		* written OK
		*/
		bool write(wxString filename);
		
		/**
		* In-memory image. Use only for UI elements
		*/
		wxImage *image;
	private:
		int width;
		int height;
		/**
		* Set to store the used colours for the palette
		*/
		ColourSet colours;
};

#endif /* pngimage_h */
