/*
 *   PharaohMapper - create minimaps from Pharaoh scenarios and saved games
 *   Copyright (C) 2007  Bianca van Schaik
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include "wx/image.h"
#include "wx/aboutdlg.h"
#include "wx/generic/aboutdlgg.h"
#include "wx/msgdlg.h"
#include "wx/busyinfo.h"

#include "pharaohmapper.h"
#include "pngimage.h"
#include "pharaohfile.h"

MyFrame   *frame = (MyFrame *) NULL;
wxBitmap  *g_Bitmap = (wxBitmap *) NULL;
PNGImage *pngimg = (PNGImage *)NULL;
wxString filename;

IMPLEMENT_APP(MyApp)

bool MyApp::OnInit(void)
{
	wxImage::AddHandler(new wxPNGHandler);

	// Create the main frame window
	frame = new MyFrame((wxFrame *) NULL, _T("PharaohMapper"), wxDefaultPosition, wxSize(200, 200));

#if wxUSE_STATUSBAR
	// Give it a status line
	frame->CreateStatusBar(1);
#endif // wxUSE_STATUSBAR

	// Make a menubar
	wxMenu *file_menu = new wxMenu;
	wxMenu *help_menu = new wxMenu;

	file_menu->Append(PHARAOHMAPPER_LOAD_FILE, _T("&Open file...\tCtrl+O"),                _T("Open a Pharaoh scenario or saved game"));
	file_menu->Append(PHARAOHMAPPER_SAVE_FILE, _T("&Save image...\tCtrl+S"),                _T("Save minimap as PNG image"));
	file_menu->Enable(PHARAOHMAPPER_SAVE_FILE, false);
	file_menu->AppendSeparator();
	file_menu->Append(PHARAOHMAPPER_QUIT, _T("E&xit"), _T("Quit program"));
	help_menu->Append(PHARAOHMAPPER_HELP, _T("&Help\tF1"), _T("Show help file"));
	help_menu->AppendSeparator();
	help_menu->Append(PHARAOHMAPPER_LICENCE, _T("&Licence"), _T("Show licence"));
	help_menu->Append(PHARAOHMAPPER_ABOUT, _T("&About PharaohMapper"), _T("About PharaohMapper"));

	wxMenuBar *menu_bar = new wxMenuBar;

	menu_bar->Append(file_menu, _T("&File"));
	menu_bar->Append(help_menu, _T("&Help"));
	
	// Associate the menu bar with the frame
	frame->SetMenuBar(menu_bar);

	PharaohCanvas *canvas = new PharaohCanvas(frame, wxPoint(0, 0), wxSize(226, 226));
	canvas->SetMinSize(wxSize(226, 226));

	// Give it scrollbars: the virtual canvas is 20 * 50 = 1000 pixels in each direction
//  canvas->SetScrollbars(20, 20, 50, 50, 4, 4);
	frame->canvas = canvas;

	frame->SetIcon(wxIcon("myicon", wxBITMAP_TYPE_ICO_RESOURCE));
	// Trick to get the minimum size depending on the canvas size:
	// force it to lay out properly, then set that size as minimum
	frame->SetInitialSize();
	frame->SetMinSize(frame->GetSize());
	frame->Show(true);

	return true;
}

BEGIN_EVENT_TABLE(MyFrame, wxFrame)
	EVT_MENU(PHARAOHMAPPER_QUIT,      MyFrame::OnQuit)
	EVT_MENU(PHARAOHMAPPER_ABOUT,     MyFrame::OnAbout)
	EVT_MENU(PHARAOHMAPPER_LOAD_FILE, MyFrame::OnLoadFile)
	EVT_MENU(PHARAOHMAPPER_SAVE_FILE, MyFrame::OnSaveFile)
	EVT_MENU(PHARAOHMAPPER_HELP,      MyFrame::OnHelp)
	EVT_MENU(PHARAOHMAPPER_LICENCE,   MyFrame::OnLicence)
END_EVENT_TABLE()

// Define my frame constructor
MyFrame::MyFrame(wxFrame *frame, const wxString& title, const wxPoint& pos, const wxSize& size):
	wxFrame(frame, wxID_ANY, title, pos, size) {
	canvas = (PharaohCanvas *) NULL;
}

// frame destructor
MyFrame::~MyFrame()
{
	if (g_Bitmap) {
		delete g_Bitmap;
		g_Bitmap = (wxBitmap *) NULL;
	}
	if (pngimg) {
		delete pngimg;
		pngimg = NULL;
	}
}

void MyFrame::OnQuit(wxCommandEvent& WXUNUSED(event)) {
	Close(true);
}

void MyFrame::OnAbout(wxCommandEvent& WXUNUSED(event)) {
	wxAboutDialogInfo about;
	about.SetName("PharaohMapper");
	about.SetVersion("1.0 (2007-02-10)");
	about.SetCopyright("Copyright (C) 2007 Bianca van Schaik a.k.a. Pecunia\n"
		"                      <pecuniam@gmail.com>"
	);
	about.SetDescription("Create minimap images from Pharaoh saved games and scenario files.\n\n"
	"This program is distributed in the hope that it will be useful,\n"
    "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
    "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
    "See the GNU General Public License or Help->Licence for more details.\n"
	);
	//about.SetWebSite("http://www.heavengames.com/angels/pecunia/citybuilding/");
	
	wxGenericAboutBox(about);
}

void MyFrame::OnHelp(wxCommandEvent& WXUNUSED(event)) {
	HelpDialog helpDialog(this, -1, "PharaohMapper Help");
	helpDialog.CentreOnParent();
	helpDialog.ShowModal();
}

void MyFrame::OnLicence(wxCommandEvent& WXUNUSED(event)) {
	LicenceDialog dialog(this, -1, "PharaohMapper Licence");
	dialog.CentreOnParent();
	dialog.ShowModal();
}

void MyFrame::OnSaveFile(wxCommandEvent& WXUNUSED(event)) {
#if wxUSE_FILEDLG
	if (!pngimg) {
		(void)wxMessageBox(_T("There's no image to save! Load a Pharaoh file first."),
			_T("Can't save nothing"), wxOK);
		return;
	}
	wxString savefile = filename;
	wxString f = wxFileSelector( wxT("Save Image"), (const wxChar *)NULL,
		savefile.RemoveLast(4),
		wxT("png"), wxT("PNG image (*.png)|*.png"),
		wxSAVE | wxOVERWRITE_PROMPT );

	if (f.empty())  return;

	if (pngimg->write(f)) {
#if wxUSE_STATUSBAR
		frame->SetStatusText(_T("Image saved"));
#endif // wxUSE_STATUSBAR
	}
#endif // wxUSE_FILEDLG
}

void MyFrame::OnLoadFile(wxCommandEvent& WXUNUSED(event)) {
#if wxUSE_FILEDLG
	// Show file selector.
	wxString f = wxFileSelector(wxT("Open Pharaoh scenario or saved game"), (const wxChar *) NULL,
		(const wxChar *) NULL, wxT("sav"),
		wxT("Pharaoh game files (*.sav, *.map)|*.sav;*.map|Pharaoh saved games (*.sav)|*.sav|Pharaoh scenarios (*.map)|*.map"),
		wxOPEN | wxFILE_MUST_EXIST);

	if (f.empty())
		return;

	// Disable the "save image" menu item
	wxMenuBar *menu = GetMenuBar();
	menu->Enable(PHARAOHMAPPER_SAVE_FILE, false);
	
	if (g_Bitmap)
		delete g_Bitmap;
	
	if (pngimg)
		delete pngimg;
	
	g_Bitmap = NULL;
	pngimg = NULL;
	canvas->Refresh();
	
	try {
		wxBusyInfo wait("Loading file...", this);
		PharaohFile sav(std::string(f.c_str()));
		pngimg = sav.getImage();
	} catch (...) {
		(void)wxMessageBox(_T("The file you selected could not be loaded.\nMake sure it is a valid Pharaoh saved game or scenario."),
			_T("Error loading file"), wxOK | wxICON_ERROR);
	}
	
	if (pngimg) {
		g_Bitmap = new wxBitmap(*(pngimg->image));
		if (!g_Bitmap->Ok()) {
			delete g_Bitmap;
			g_Bitmap = (wxBitmap *) NULL;
		} else {
			// Set filename for use in file save dialog
			filename = f;
			// Enable "save image" again
			menu->Enable(PHARAOHMAPPER_SAVE_FILE, true);
#if wxUSE_STATUSBAR
			frame->SetStatusText(_T("Done"));
#endif // wxUSE_STATUSBAR
		}
	}

	canvas->Refresh();
#endif // wxUSE_FILEDLG
}

BEGIN_EVENT_TABLE(PharaohCanvas, wxScrolledWindow)
	EVT_PAINT(PharaohCanvas::OnPaint)
END_EVENT_TABLE()

/**
* Default constructor. Calls parent constructur and does nothing.
*/
PharaohCanvas::PharaohCanvas(wxWindow *parent, const wxPoint& pos, const wxSize& size):
	wxScrolledWindow(parent, wxID_ANY, pos, size)
{
}

/**
* Repaints the canvas
*/
void PharaohCanvas::OnPaint(wxPaintEvent& WXUNUSED(event)) {
	wxPaintDC dc(this);

	if (g_Bitmap && g_Bitmap->Ok()) {
		wxMemoryDC memDC;
		memDC.SelectObject(* g_Bitmap);

		// Normal, non-transparent blitting
		dc.Blit(0, 0, g_Bitmap->GetWidth(), g_Bitmap->GetHeight(), & memDC, 0, 0, wxCOPY, false);

		memDC.SelectObject(wxNullBitmap);
	}

}

/*****************************
 * HELPDIALOG IMPLEMENTATION *
 *****************************/

HelpDialog::HelpDialog(wxWindow *parent, wxWindowID id,
	const wxString &title, const wxPoint &position,
	const wxSize& size, long style):
	wxDialog( parent, id, title, position, size, style)
{
	wxString text = 
	"PharaohMapper is a small program to create minimaps from adventures and saved games, exactly like the in-game minimap. "
	"This program supports files from both Pharaoh and Cleopatra.\n\n"
	"To load a file, use the File -> Open dialog. "
	"If you did a default installation, the location for Pharaoh scenarios is:\n"
	"C:\\Sierra\\Pharaoh\\Maps\n\n"
	"For Pharaoh saved games, the default folder is:\n"
	"C:\\Sierra\\Pharaoh\\Save\\YOUR_NAME\n\n"
	"Once you have selected a file, the minimap for that file will appear after a short while.\n\n"
	"You can then save the minimap as a PNG image using File -> Save. The image is saved in 256-colour format to make the file as small as possible.";
	
	wxBoxSizer *topsizer = new wxBoxSizer(wxVERTICAL);
	wxTextCtrl *dialogText = new wxTextCtrl( this, -1, text, wxDefaultPosition,
		wxSize(280, 280), wxTE_MULTILINE | wxTE_READONLY);
	dialogText->SetInsertionPoint(0); // make sure the top part is visible
	topsizer->Add(dialogText, 1, wxEXPAND | wxALL, 10);
	
	wxBoxSizer *button_sizer = new wxBoxSizer(wxHORIZONTAL);
	wxButton * b = new wxButton( this, wxID_OK, "OK");
	button_sizer->Add(b, 0, wxBOTTOM, 10);
	b->SetFocus();
	topsizer->Add(button_sizer, 0, wxALIGN_CENTER);
	
	SetSizer(topsizer);
	topsizer->SetSizeHints(this);
}

