/*
 *   ZeusMapper - create minimaps from Zeus scenarios and saved games
 *   Copyright (C) 2007  Bianca van Schaik
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include "wx/image.h"
#include "wx/aboutdlg.h"
#include "wx/generic/aboutdlgg.h"
#include "wx/msgdlg.h"
#include "wx/busyinfo.h"

#include "zeusmapper.h"
#include "pngimage.h"
#include "zeusfile.h"

MyFrame   *frame = (MyFrame *) NULL;
wxBitmap  *g_Bitmap = (wxBitmap *) NULL;
PNGImage *pngimg = (PNGImage *)NULL;
PNGImage **images = new PNGImage*[ZeusFile::MAX_MAPS];
wxString filename;

IMPLEMENT_APP(MyApp)

bool MyApp::OnInit(void)
{
	// Initialise image array
	for (int i = 0; i < ZeusFile::MAX_MAPS; i++) {
		images[i] = NULL;
	}
	
	wxImage::AddHandler(new wxPNGHandler);

	// Create the main frame window
	frame = new MyFrame((wxFrame *) NULL, _T("ZeusMapper"), wxDefaultPosition, wxSize(200, 200));

#if wxUSE_STATUSBAR
	// Give it a status line
	frame->CreateStatusBar(1);
#endif // wxUSE_STATUSBAR

	// Make a menubar
	wxMenu *file_menu = new wxMenu;
	wxMenu *help_menu = new wxMenu;
	wxMenu *show_menu = new wxMenu;

	file_menu->Append(ZEUSMAPPER_LOAD_FILE, _T("&Open file...\tCtrl+O"), _T("Open a Zeus adventure or saved game"));
	file_menu->Append(ZEUSMAPPER_SAVE_FILE, _T("&Save image...\tCtrl+S"), _T("Save minimap as PNG image"));
	file_menu->Enable(ZEUSMAPPER_SAVE_FILE, false);
	file_menu->AppendSeparator();
	file_menu->Append(ZEUSMAPPER_QUIT, _T("E&xit"), _T("Quit program"));
	help_menu->Append(ZEUSMAPPER_HELP, _T("&Help\tF1"), _T("Show help file"));
	help_menu->AppendSeparator();
	help_menu->Append(ZEUSMAPPER_LICENCE, _T("&Licence"), _T("Show licence"));
	help_menu->Append(ZEUSMAPPER_ABOUT, _T("&About ZeusMapper"), _T("About ZeusMapper"));

	wxMenuBar *menu_bar = new wxMenuBar;

	menu_bar->Append(file_menu, _T("&File"));
	menu_bar->Append(show_menu, _T("&Show"));
	menu_bar->Append(help_menu, _T("&Help"));
	
	// Associate the menu bar with the frame
	frame->SetMenuBar(menu_bar);

	// Disable the "show" menu
	menu_bar->EnableTop(1, false);
	
	//wxNotebook *tabbedpane = new wxNotebook(frame, -1);
	ZeusCanvas *canvas = new ZeusCanvas(frame, wxPoint(0, 0), wxSize(226, 226));
	canvas->SetMinSize(wxSize(226, 226));
	
	// Give it scrollbars: the virtual canvas is 20 * 50 = 1000 pixels in each direction
//  canvas->SetScrollbars(20, 20, 50, 50, 4, 4);
	frame->canvas = canvas;
//	tabbedpane->SetInitialSize();
	//wxMessageBox("size: );
	frame->SetIcon(wxIcon("myicon", wxBITMAP_TYPE_ICO_RESOURCE));
	// Trick to get the minimum size depending on the canvas size:
	// force it to lay out properly, then set that size as minimum
	frame->SetInitialSize();
	frame->SetMinSize(frame->GetSize());
	frame->Show(true);

	return true;
}

BEGIN_EVENT_TABLE(MyFrame, wxFrame)
	EVT_MENU(ZEUSMAPPER_QUIT,      MyFrame::OnQuit)
	EVT_MENU(ZEUSMAPPER_ABOUT,     MyFrame::OnAbout)
	EVT_MENU(ZEUSMAPPER_LOAD_FILE, MyFrame::OnLoadFile)
	EVT_MENU(ZEUSMAPPER_SAVE_FILE, MyFrame::OnSaveFile)
	EVT_MENU(ZEUSMAPPER_HELP,      MyFrame::OnHelp)
	EVT_MENU(ZEUSMAPPER_LICENCE,   MyFrame::OnLicence)
	EVT_MENU_RANGE(
		ZEUSMAPPER_SHOW_BASE,
		ZEUSMAPPER_SHOW_BASE + ZeusFile::MAX_MAPS,
		MyFrame::OnSelectMap)
END_EVENT_TABLE()

// Define my frame constructor
MyFrame::MyFrame(wxFrame *frame, const wxString& title, const wxPoint& pos, const wxSize& size):
	wxFrame(frame, wxID_ANY, title, pos, size) {
	canvas = (ZeusCanvas *) NULL;
}

// frame destructor
MyFrame::~MyFrame()
{
	if (g_Bitmap) {
		delete g_Bitmap;
		g_Bitmap = (wxBitmap *) NULL;
	}
	for (int i = 0; i < ZeusFile::MAX_MAPS; i++) {
		if (images[i]) {
			delete images[i];
			images[i] = NULL;
		}
	}
	delete images;
}

void MyFrame::OnQuit(wxCommandEvent& WXUNUSED(event)) {
	Close(true);
}

void MyFrame::OnAbout(wxCommandEvent& WXUNUSED(event)) {
	wxAboutDialogInfo about;
	about.SetName("ZeusMapper");
	about.SetVersion("1.0 (2007-01-04)");
	about.SetCopyright("Copyright (C) 2007 Bianca van Schaik a.k.a. Pecunia\n"
		"                      <pecuniam@gmail.com>"
	);
	about.SetDescription("Create minimap images from Zeus saved games and adventure files.\n\n"
	"This program is distributed in the hope that it will be useful,\n"
    "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
    "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
    "See the GNU General Public License or Help->Licence for more details.\n"
	);
	//about.SetWebSite("http://www.heavengames.com/angels/pecunia/citybuilding/");
	
	wxGenericAboutBox(about);
}

void MyFrame::OnHelp(wxCommandEvent& WXUNUSED(event)) {
	HelpDialog helpDialog(this, -1, "ZeusMapper Help");
	helpDialog.CentreOnParent();
	helpDialog.ShowModal();
}

void MyFrame::OnLicence(wxCommandEvent& WXUNUSED(event)) {
	LicenceDialog dialog(this, -1, "ZeusMapper Licence");
	dialog.CentreOnParent();
	dialog.ShowModal();
}

void MyFrame::OnSaveFile(wxCommandEvent& WXUNUSED(event)) {
#if wxUSE_FILEDLG
	if (!pngimg) {
		(void)wxMessageBox(_T("There's no image to save! Load a Zeus file first."),
			_T("Can't save nothing"), wxOK);
		return;
	}
	wxString savefile = filename;
	wxString f = wxFileSelector( wxT("Save Image"), (const wxChar *)NULL,
		savefile.RemoveLast(4),
		wxT("png"), wxT("PNG image (*.png)|*.png"),
		wxSAVE | wxOVERWRITE_PROMPT );

	if (f.empty())  return;

	if (pngimg->write(f)) {
#if wxUSE_STATUSBAR
		frame->SetStatusText(_T("Image saved"));
#endif // wxUSE_STATUSBAR
	}
#endif // wxUSE_FILEDLG
}

void MyFrame::OnLoadFile(wxCommandEvent& WXUNUSED(event)) {
#if wxUSE_FILEDLG
	// Show file selector.
	wxString f = wxFileSelector(wxT("Open Zeus adventure or saved game"), (const wxChar *) NULL,
		(const wxChar *) NULL, wxT("sav"),
		wxT("Zeus game files (*.sav, *.pak, *.map)|*.sav;*.pak;*.map|Zeus saved games (*.sav)|*.sav|Zeus adventures (*.pak)|*.pak|Zeus map files (*.map)|*.map"),
		wxOPEN | wxFILE_MUST_EXIST);

	if (f.empty())
		return;

	// Disable the "save image" menu item & the whole "show" menu
	wxMenuBar *menu = GetMenuBar();
	menu->Enable(ZEUSMAPPER_SAVE_FILE, false);
	menu->EnableTop(1, false);
	
	if (g_Bitmap)
		delete g_Bitmap;
	
	for (int i = 0; i < ZeusFile::MAX_MAPS; i++) {
		if (images[i]) {
			delete images[i];
			images[i] = NULL;
		}
	}
	
	g_Bitmap = NULL;
	pngimg = NULL;
	canvas->Refresh();
	
	int num = 0, loaded = 0;
	try {
		wxBusyInfo wait("Loading file...", this);
		ZeusFile sav(std::string(f.c_str()));
		num = sav.getNumMaps();
		for (int i = 0; i < num; i++) {
			images[i] = sav.getImage();
			loaded++;
		}
	} catch (...) {
		(void)wxMessageBox(_T("The file you selected could not be loaded.\nMake sure it is a valid Zeus saved game or adventure."),
			_T("Error loading file"), wxOK | wxICON_ERROR);
	}
	
	if (loaded) {
		pngimg = images[0];
		g_Bitmap = new wxBitmap(*(pngimg->image));
		if (!g_Bitmap->Ok()) {
			delete g_Bitmap;
			g_Bitmap = (wxBitmap *) NULL;
		} else {
			// Set filename for use in file save dialog
			filename = f;
			// Enable "save image" again
			menu->Enable(ZEUSMAPPER_SAVE_FILE, true);
			
			if (loaded > 1) {
				// Replace the "show" menu
				wxMenu *show_menu = new wxMenu;
				wxString menuItems[] = { "Parent city",
					"Colony 1", "Colony 2", "Colony 3", "Colony 4" };
				for (int i = 0; i < loaded; i++) {
					show_menu->AppendRadioItem(ZEUSMAPPER_SHOW_BASE + i,
						menuItems[i], "Show minimap for " + menuItems[i]);
				}
				show_menu = menu->Replace(1, show_menu, _T("&Show"));
				menu->EnableTop(1, true);
				delete show_menu;
#if wxUSE_STATUSBAR
				wxString str;
				str.Printf(_T("Done. %d maps loaded."), loaded);
				frame->SetStatusText(str);
#endif // wxUSE_STATUSBAR
			} else {
#if wxUSE_STATUSBAR
				frame->SetStatusText(_T("Done."));
#endif // wxUSE_STATUSBAR
			}

		}
	}

	canvas->Refresh();
#endif // wxUSE_FILEDLG
}

void MyFrame::OnSelectMap(wxCommandEvent& evt) {
	int selection = evt.GetId() - ZEUSMAPPER_SHOW_BASE;

	if (images[selection]) {
		if (g_Bitmap)
			delete g_Bitmap;
		
		pngimg = images[selection];
		g_Bitmap = new wxBitmap(*(pngimg->image));
		if (!g_Bitmap->Ok()) {
			delete g_Bitmap;
			g_Bitmap = (wxBitmap *) NULL;
		}
	}
	canvas->Refresh();
}

BEGIN_EVENT_TABLE(ZeusCanvas, wxScrolledWindow)
	EVT_PAINT(ZeusCanvas::OnPaint)
END_EVENT_TABLE()

/**
* Default constructor. Calls parent constructor and does nothing.
*/
ZeusCanvas::ZeusCanvas(wxWindow *parent, const wxPoint& pos, const wxSize& size):
	wxScrolledWindow(parent, wxID_ANY, pos, size)
{
}

/**
* Repaints the canvas
*/
void ZeusCanvas::OnPaint(wxPaintEvent& WXUNUSED(event)) {
	wxPaintDC dc(this);

	if (g_Bitmap && g_Bitmap->Ok()) {
		wxMemoryDC memDC;
		memDC.SelectObject(* g_Bitmap);

		// Normal, non-transparent blitting
		dc.Blit(0, 0, g_Bitmap->GetWidth(), g_Bitmap->GetHeight(), & memDC, 0, 0, wxCOPY, false);

		memDC.SelectObject(wxNullBitmap);
	}

}

/*****************************
 * HELPDIALOG IMPLEMENTATION *
 *****************************/

HelpDialog::HelpDialog(wxWindow *parent, wxWindowID id,
	const wxString &title, const wxPoint &position,
	const wxSize& size, long style):
	wxDialog( parent, id, title, position, size, style)
{
	wxString text = 
	"ZeusMapper is a small program to create minimaps from adventures and saved games, exactly like the in-game minimap. "
	"This program supports files from both Zeus and Poseidon.\n\n"
	"To load a file, use the File -> Open dialog. "
	"If you did a default installation, the location for Zeus adventures is:\n"
	"C:\\Impressions Games\\Zeus\\Adventures\n\n"
	"For Zeus saved games, the default folder is:\n"
	"C:\\Impressions Games\\Zeus\\Save\\YOUR_NAME\n\n"
	"Once you have selected a file, the minimap for that file will appear after a short while. "
	"For adventures, both the parent city minimap and the colony maps, if any, are loaded. If there are colony maps available, the \"Show\" menu will be enabled so you can view and save the colony minimaps.\n\n"
	"You can then save the minimap as a PNG image using File -> Save. The image is saved in 256-colour format to make the file as small as possible.";
	
	wxBoxSizer *topsizer = new wxBoxSizer(wxVERTICAL);
	wxTextCtrl *dialogText = new wxTextCtrl( this, -1, text, wxDefaultPosition,
		wxSize(300, 300), wxTE_MULTILINE | wxTE_READONLY);
	dialogText->SetInsertionPoint(0); // make sure the top part is visible
	topsizer->Add(dialogText, 1, wxEXPAND | wxALL, 10);
	
	wxBoxSizer *button_sizer = new wxBoxSizer(wxHORIZONTAL);
	wxButton * b = new wxButton( this, wxID_OK, "OK");
	button_sizer->Add(b, 0, wxBOTTOM, 10);
	b->SetFocus();
	topsizer->Add(button_sizer, 0, wxALIGN_CENTER);
	
	SetSizer(topsizer);
	topsizer->SetSizeHints(this);
}

